#
#  Copyright 2010-2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
#  This file is licensed under the Apache License, Version 2.0 (the "License").
#  You may not use this file except in compliance with the License. A copy of
#  the License is located at
# 
#  http://aws.amazon.com/apache2.0/
# 
#  This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
#  CONDITIONS OF ANY KIND, either express or implied. See the License for the
#  specific language governing permissions and limitations under the License.
#
import boto3
import sys
import time
import argparse
import re
# from botocore.exceptions import ClientError


def createPreTokenStack(stack_params):
    # read the contents of the Cloudformation yaml file
    f = open(stack_params["cloudFormation"], "r")
    templateBody = f.read()

    # create a cloudformation client
    client = boto3.client('cloudformation')

    # use the Cloudformation client method create_stack to create the Cloudformation stack and return a response
    create_stack_response = client.create_stack(
        StackName = stack_params["stackName"],
        TemplateBody = templateBody,
        Parameters=[
            {
                'ParameterKey': 'UserPoolName',
                'ParameterValue': stack_params["userPoolName"],
            },
            {
                'ParameterKey': 'UserPoolClientName',
                'ParameterValue': stack_params["userPoolClientName"],
            },
            {
                'ParameterKey': 'UserPoolDomainName',
                'ParameterValue': stack_params["userPoolDomainName"],
            },
            {
                'ParameterKey': 'DynamoDBTableName',
                'ParameterValue': stack_params["tableName"],
            }
        ],
        NotificationARNs=[],
        Capabilities=[
            'CAPABILITY_NAMED_IAM',
        ],
        OnFailure='ROLLBACK'
    )

    # While the Cloudformation stack is being built. check for events such as CREATE_COMPLETE and ROLLBACK_COMPLETE
    # to inform the user if the stack has been built successfully or if the stack has failed and rolled back.
    stack_building = True
    print("Stack creation in process...")
    try_count = 0
    while stack_building is True:
        try_count += 1
        event_list = client.describe_stack_events(
            StackName=stack_params["stackName"]).get("StackEvents")
        stack_event = event_list[0]

        if (stack_event.get('ResourceType') == 'AWS::CloudFormation::Stack' and
                stack_event.get('ResourceStatus') == 'CREATE_COMPLETE'):
            stack_building = False
            print("Stack construction complete.")
        elif (stack_event.get('ResourceType') == 'AWS::CloudFormation::Stack' and
            stack_event.get('ResourceStatus') == 'ROLLBACK_COMPLETE'):
            stack_building = False
            print("Stack construction failed.")
            sys.exit(1)
        else:
            print("Stack building . . .")
            time.sleep(10)

    # Use the decribe_stack_response method to describe the stack that was created
    describe_stack_response = client.describe_stacks(StackName=stack_params["stackName"])

    # retrieve the list of stacks
    stacks = describe_stack_response['Stacks']
    list_of_outputs = {}
    # Iterate over the list of stacks and extract the outputs for the following:
    # CognitoUserPoolClientId
    # CongnitoUserPoolId
    # DynamoDBTableName
    # StackArn
    # The above Output values will be displayed on the console or terminal when the stack creation completes.
    # Please note them down from the console or terminal if necessary
    for stack in stacks:
        outputs = stack['Outputs']
        for output in outputs:
            list_of_outputs[output['OutputKey']] = output['OutputValue']
            if output['OutputKey'] == 'CongnitoUserPoolId' :
              client_pool = output['OutputValue']
            if output['OutputKey'] == 'CognitoUserPoolClientId' :
              client_id = output['OutputValue']

    return list_of_outputs

def get_callback_url(client_pool, client_id):

    client = boto3.client('cognito-idp')
    response = client.describe_user_pool_client(
    UserPoolId= client_pool,
    ClientId= client_id
    )

    user_pool_client = response["UserPoolClient"]
    callback_urls = user_pool_client["CallbackURLs"]
    callback_url = ''
    for url in callback_urls:
        callback_url = url

    return callback_url


