#
#  Copyright 2010-2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
#  This file is licensed under the Apache License, Version 2.0 (the "License").
#  You may not use this file except in compliance with the License. A copy of
#  the License is located at
# 
#  http://aws.amazon.com/apache2.0/
# 
#  This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
#  CONDITIONS OF ANY KIND, either express or implied. See the License for the
#  specific language governing permissions and limitations under the License.
#
from subprocess import Popen, PIPE, STDOUT
import boto3
from botocore.exceptions import ClientError

def createPreTokenUsers(createUser_args):
    client = boto3.client('cognito-idp')
    sessionid= ""

    # Step1 : Create a new user
    try:
        response = client.admin_create_user(
            UserPoolId=createUser_args["CongnitoUserPoolId"],
            Username=createUser_args["usrname"],
            UserAttributes=[
                {
                    'Name': 'email',
                    'Value': createUser_args["email"]
                },
                {
                    'Name': 'email_verified',
                    'Value': 'true'
                }
            ],
            TemporaryPassword=createUser_args["pwd"],
            MessageAction='SUPPRESS'
        )
    except ClientError as e:
        print("Error creating a new user")
        raise e

    # Step2 : Get a session id to enable user
    try:
        response = client.admin_initiate_auth(
            UserPoolId=createUser_args["CongnitoUserPoolId"],
            ClientId=createUser_args["CognitoUserPoolClientId"],
            AuthFlow='ADMIN_USER_PASSWORD_AUTH',
            AuthParameters={
                'USERNAME': createUser_args["usrname"],
                'PASSWORD': createUser_args["pwd"]
            }
        )

        sessionid = response['Session']

    except ClientError as e:
        print("Error in getting a session id to enable user")
        raise e

    # Step3 : Use the session from last step to enable the user and give a new password
    response = client.admin_respond_to_auth_challenge(
        UserPoolId=createUser_args["CongnitoUserPoolId"],
        ClientId=createUser_args["CognitoUserPoolClientId"],
        ChallengeName='NEW_PASSWORD_REQUIRED',
        ChallengeResponses={
            'NEW_PASSWORD': createUser_args["pwd"],
            'USERNAME': createUser_args["usrname"]
        },
        Session=sessionid,
    )

    return

def configurePreTokenGenerationTrigger(configurePreTokenGenerationLambda_input):
    client = boto3.client('cognito-idp')

    try:
        response = client.update_user_pool(
            UserPoolId=configurePreTokenGenerationLambda_input["CongnitoUserPoolId"],
            LambdaConfig={
                'PreTokenGeneration': configurePreTokenGenerationLambda_input["lambdaFunctionArn"]
            }
        )
    except ClientError as e:
        print("Error configuring pre token generation trigger")
        raise e

    return

def createUserPoolDomain(createUserPoolDomain_input):
    client = boto3.client('cognito-idp')

    try:
        response = client.create_user_pool_domain(
            Domain=createUserPoolDomain_input["domain"],
            UserPoolId=createUserPoolDomain_input["CongnitoUserPoolId"],
        )
    except ClientError as e:
        print("Error creating a new user pool domain")
        raise e

    return

def createAppClient(createAppClient_input):
    client = boto3.client('cognito-idp')

    try:
        response = client.create_user_pool_client(
            UserPoolId=createAppClient_input['CongnitoUserPoolId'],
            ClientName='string',
            GenerateSecret=True,
            RefreshTokenValidity=123,
            ReadAttributes=[
                'string',
            ],
            WriteAttributes=[
                'string',
            ],
            ExplicitAuthFlows=[
                'ALLOW_CUSTOM_AUTH'|'ALLOW_USER_SRP_AUTH'|'ALLOW_REFRESH_TOKEN_AUTH',
            ],
            PreventUserExistenceErrors='LEGACY'|'ENABLED',
            SupportedIdentityProviders=[
                'string',
            ],
            CallbackURLs=[
                'https://www.example.com/',
            ],
            LogoutURLs=[
                'https://www.example.com/',
            ],
            DefaultRedirectURI='string',
            AllowedOAuthFlows=[
                'code'|'implicit',
            ],
            AllowedOAuthScopes=[
                'phone', 'email', 'aws.cognito.signin.user.admin'
            ],
        )
    except ClientError as e:
        print("Error creating a new user pool domain")
        raise e

    return
